@@ -25,71 +25,63 @@
 /**         Test case, catalog, plan creation         */
 /******************************************************/
 
-function creaTestCatalog(path) {
-	var tcInput = document.getElementById("catName");
-	var catalogName = tcInput.value;
-    
-    if (catalogName == null || catalogName.length == 0) {
-		document.getElementById('catErrorMsgSpan').innerHTML = _("You must specify a name. Length between 4 and 90 characters.");
-    } else {
-    	var catName = stripLessSpecialChars(catalogName);
-    	
-    	if (catName.length > 90 || catName.length < 4) {
-    		document.getElementById('catErrorMsgSpan').innerHTML = _("Length between 4 and 90 characters.");
-    	} else { 
-    		document.getElementById('catErrorMsgSpan').innerHTML = ''; 
-    		var url = baseLocation+"/testcreate?type=catalog&path="+path+"&title="+catName;
-    		window.location = url;
-    	}
-    }
-}
+(function($) {
+		$(function() {
 
-function creaTestCase(catName) { 
-	var tcInput = document.getElementById('tcName');
-	var testCaseName = tcInput.value; 
+            $( "#createTestCatalogForm" ).submit(function( event ) {
+                var form = $(this);
+                var testCatalogName = $("input[name='title']", form).val();
+                
+                if (testCatalogName == null || testCatalogName.length < 4) {
+                    $('#catErrorMsgSpan').html(_("You must specify a name. Minimum length is 4 characters."));
+                    event.preventDefault();
+                } else {
+                    $('#catErrorMsgSpan').html('');
+                }
 
-    if (testCaseName == null || testCaseName.length == 0) {
-		document.getElementById('errorMsgSpan').innerHTML = _("You must specify a name. Length between 4 and 90 characters.");
-    } else {
-    	var tcName = stripLessSpecialChars(testCaseName); 
-    	
-    	if (tcName.length > 90 || tcName.length < 4) {
-    		document.getElementById('errorMsgSpan').innerHTML = _("Length between 4 and 90 characters.");
-    	} else { 
-    		document.getElementById('errorMsgSpan').innerHTML = ''; 
-    		var url = baseLocation+"/testcreate?type=testcase&path="+catName+"&title="+tcName;
-    		window.location = url;
-    	}
-    }
-}
+                return;
+            });
 
-function creaTestPlan(catName) { 
-	var planInput = document.getElementById('planName');
-	var testPlanName = planInput.value; 
 
-    if (testPlanName == null || testPlanName.length == 0) {
-		document.getElementById('errorMsgSpan2').innerHTML = _("You must specify a name. Length between 4 and 90 characters.");
-    } else {
-    	var tplanName = stripLessSpecialChars(testPlanName); 
-    	
-    	if (tplanName.length > 90 || tplanName.length < 4) {
-    		document.getElementById('errorMsgSpan2').innerHTML = _("Length between 4 and 90 characters.");
-    	} else { 
-    		document.getElementById('errorMsgSpan2').innerHTML = ''; 
-            (function($) {
-                $(function() {
-                    $("#dialog_testplan").dialog({width: 640, height: 430, modal: true});
-                });
-            })(jQuery_testmanager);	
-    	}
-    }
-}
+            $( "#createTestCaseForm" ).submit(function( event ) {
+                var form = $(this);
+                var testCaseName = $("input[name='title']", form).val();
+                
+                if (testCaseName == null || testCaseName.length < 4) {
+                    $('#errorMsgSpan').html(_("You must specify a name. Minimum length is 4 characters."));
+                    event.preventDefault();
+                } else {
+                    $('#errorMsgSpan').html('');
+                }
+
+                return;
+            });
+
+
+            $( "#createTestPlanForm" ).submit(function( event ) {
+                var form = $(this);
+                
+                if ($("input[name='mustSubmit']", form).val() === 'false') {
+                    var testPlanName = $("input[name='title']", form).val();
+
+                    if (testPlanName == null || testPlanName.length < 4) {
+                        $('#errorMsgSpan2').html(_("You must specify a name. Minimum length is 4 characters."));
+                    } else {
+                        $('#errorMsgSpan2').html('');
+
+                        $("#dialog_testplan").dialog({width: 640, height: 430, modal: true});
+                    }
+
+                    event.preventDefault();
+                }
+                    
+                return;
+            });
+		});
+})(jQuery_testmanager);
 
 function createTestPlanConfirm(catName) {
-    var planInput = document.getElementById('planName');
-    var testPlanName = planInput.value; 
-    var tplanName = stripLessSpecialChars(testPlanName); 
-
+    var testPlanName = $('#createTestPlanForm').find('input[name="title"]').val();
     var testplanContainsAll = "true";
     var testplanSnapshot = "false";
 
@@ -117,11 +109,17 @@ function createTestPlanConfirm(catName) {
 
     var selectedTestCases = "";
     if (testplanContainsAll == 'false') {
-        selectedTestCases = "&selectedTCs="+getSelectedTestCases();
+        selectedTestCases = getSelectedTestCases();
     }
-    
-    var url = baseLocation+"/testcreate?type=testplan&path="+catName+"&containsAll="+testplanContainsAll+"&snapshot="+testplanSnapshot+"&title="+tplanName+selectedTestCases;
-    window.location = url;
+
+    var form = $('#createTestPlanForm');
+
+    $("input[name='containsAll']", form).val(testplanContainsAll);
+    $("input[name='snapshot']", form).val(testplanSnapshot);
+    $("input[name='selectedTCs']", form).val(selectedTestCases);
+
+    $("input[name='mustSubmit']", form).val('true');
+    $('#createTestPlanForm').submit();
 }
 
 function createTestPlanCancel() {
@@ -187,15 +185,36 @@ function removeTestCase(tcName, planId) {
 	}
 }
 
-function creaTicket(tcName, planId, planName, summary){
-    var tokens = $('span[name=breadcrumb]').map(function() { return this.innerHTML; }).get();
+function creaTicket(tcName, planId, planName, summary) {
     var fullSummary = "";
     
-    for (i=1; i<tokens.length; i++) {
-        fullSummary += tokens[i] + " - ";
+    switch(ticketSummaryOption) {
+        case 'empty':
+            break;
+
+        case 'fixed_text':
+            fullSummary = ticketSummaryText + ticketSummarySeparator;
+            break;
+
+        case 'last_n_catalogs':
+        case 'full_path':
+        default:
+            var tokens = $('span[name=breadcrumb]').map(function() { return this.innerHTML; }).get();
+
+            var start = 1;
+            
+            if (ticketSummaryOption == 'last_n_catalogs' && ticketSummaryNumCatalogs < tokens.length) {
+                start = tokens.length - ticketSummaryNumCatalogs + 1;
+            }
+            
+            for (var i=start; i<tokens.length; i++) {
+                fullSummary += tokens[i] + ticketSummarySeparator;
+            }
+        
+            fullSummary += summary + ticketSummarySeparator;
     }
-    
-    fullSummary += summary + " - " + _("[Insert problem summary]");
+
+    fullSummary += _("[Insert problem summary]");
 
 	var url = baseLocation+'/newticket?testcaseid='+tcName+'&planid='+planId+'&planname='+planName+'&summary='+stripLessSpecialChars(fullSummary)+'&description=Test%20Case:%20[wiki:'+tcName+'?planid='+planId+'],%20Test%20Plan:%20'+planName+'%20('+planId+')'; 
 	window.location = url;
@@ -708,13 +727,11 @@ function deselectRow(tableId, node) {
 }
 
 function showPencil(id) {
-    el = document.getElementById(id);
-    el.style.display = '';
+    $('#'+id).show();
 }
 
 function hidePencil(id) {
-    el = document.getElementById(id);
-    el.style.display = 'none';
+    $('#'+id).hide();
 }
 
 /******************************************************/
@@ -739,10 +756,10 @@ function changestate(tc, planid, path, newStatus, newStatusColor, newLabel) {
         
         displayNode("tcTitleStatusIcon"+currStatusColor, false);
 		
-        document.getElementById("tcTitleStatusIcon"+newStatusColor).title = newLabel;
+        $("#tcTitleStatusIcon"+newStatusColor).prop('title', newLabel);
         displayNode("tcTitleStatusIcon"+newStatusColor, true);
 		
-        currStatus = newStatus; 
+        currStatus = newStatus;
         currStatusColor = newStatusColor;
     } else {
         (function($) {
@@ -992,9 +1009,18 @@ function stripLessSpecialChars(str) {
 }
 
 function displayNode(id, show, mode) {
-    var msgNode = document.getElementById(id);
-    if (msgNode) {
-       msgNode.style.display = show ? (mode ? mode : "block") : "none";
+    if (show) {
+        $('#'+id).show();
+    } else {
+        $('#'+id).hide();
+    }
+}
+
+function displayNodes(name, show, mode) {
+    if (show) {
+        $("[name='"+name+"']").show();
+    } else {
+        $("[name='"+name+"']").hide();
     }
 }
 
@@ -1030,22 +1056,38 @@ function doAjaxCall(url, method, params) {
     return responseText;
 }
 
-function editField(name) {
-    displayNode('custom_field_value_'+name, false);
-    displayNode('custom_field_'+name, true);
-    displayNode('update_button_'+name, true);
+var editingFields = {};
+
+function showEditingPencil(name) {
+    if (!editingFields[name]) {
+        showPencil("field_pencilIcon"+name);
+    }
 }
 
-function sendUpdate(realm, name) {
+function editField(name, type) {
+    displayNode('custom_field_value_'+name, false);
+    displayNode('container_custom_field_'+name, true);
+    displayNode('update_button_'+name, true);
+    editingFields[name] = true;
+    hidePencil("field_pencilIcon"+name);
+}
+
+function sendUpdate(realm, name, type) {
    	var objKeyField = document.getElementById("obj_key_field");
     var objKey = objKeyField.value;
 
    	var objPropsField = document.getElementById("obj_props_field");
     var objProps = objPropsField.value;
 
-   	var inputField = document.getElementById("custom_field_"+name);
-	var value = inputField.value;
-    
+    var value = null;
+    if (type == 'radio') {
+        value = $("input:radio[name='custom_field_"+name+"']:checked").val();
+    } else if (type == 'checkbox') {
+        value = $('#custom_field_'+name).checked() ? '1' : '0';
+    } else {
+        value = $('#custom_field_'+name).val();
+    }
+
     var url = baseLocation+"/propertyupdate";
 
     /*
@@ -1056,15 +1098,21 @@ function sendUpdate(realm, name) {
     result = doAjaxCall(url, "GET", params);
 
     /*
-	Handle errors in the Ajax call
+	Handle result of the Ajax call
 	*/
     if (result == 'OK') {
-        var readonlyField = document.getElementById("custom_field_value_"+name);
-        readonlyField.innerHTML = value;
+        if (type == 'textarea') {
+            value = replaceAll('\n\r', '<br />', value);
+            value = replaceAll('\n', '<br />', value);
+        }
+        
+        $("#custom_field_value_"+name).html(value);
 
         displayNode('custom_field_value_'+name, true);
-        displayNode('custom_field_'+name, false);
+        displayNode('container_custom_field_'+name, false);
         displayNode('update_button_'+name, false);
+        delete editingFields[name];
+
     } else {
         (function($) {
             $(function() {
@@ -1106,6 +1154,14 @@ function loadMessageCatalog() {
 	include('../chrome/testmanager/js/' + lc + '.js');
 }
 
+function escapeRegExp(str) {
+  return str.replace(/[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]/g, "\\$&");
+}
+
+function replaceAll(find, replace, str) {
+  return str.replace(new RegExp(escapeRegExp(find), 'g'), replace);
+}
+
 /**
  * Adds the specified function, by name or by pointer, to the window.onload() queue.
  * 
