@@ -331,6 +331,18 @@ class TestCatalog(AbstractTestDescription):
     def create_instance(self, key):
         return TestCatalog(self.env, key['id'])
 
+    @classmethod
+    def list_root_catalogs(cls, env):
+        """
+        Returns a list of the root-level catalogs.
+        """
+        tc_search = cls(env)
+        tc_search['page_name'] ='TC_TT%'
+        
+        for tc in tc_search.list_matching_objects(exact_match=False):
+            # Only return root catalogs
+            if tc['page_name'].count('_') == 1:
+                yield tc
    
 class TestCase(AbstractTestDescription):
 
@@ -1054,6 +1066,13 @@ class TestManagerModelProvider(Component):
                     self.config.set('test-outcomes', 'default', 'TO_BE_TESTED')
                     config_dirty = True
 
+                if 'testmanager' not in self.config:
+                    self.config.set('testmanager', 'ticket_summary_option', 'full_path')
+                    self.config.set('testmanager', 'ticket_summary_text', '')
+                    self.config.set('testmanager', 'ticket_summary_num_catalogs', '1')
+                    self.config.set('testmanager', 'ticket_summary_separator', ' - ')
+                    config_dirty = True
+
                 if config_dirty:
                     self.config.save()
 
