@@ -463,7 +463,7 @@ class TestStatsPlugin(Component):
 
             data = {}
             data['resolution'] = str(graph_res)
-            data['baseurl'] = req.base_url
+            data['baseurl'] = req.href()
             data['testplans'] = testplan_list
             data['ctestplan'] = testplan
 
@@ -528,9 +528,6 @@ def daterange(begin, end, delta = timedelta(1)):
      end   -- a date (or datetime) object; the end of the range.
      delta -- (optional) a timedelta object; how much to step each iteration.
                  Default step is 1 day.
-                 
-     Usage:
-
      """
      if not isinstance(delta, timedelta):
           delta = timedelta(delta)
