@@ -60,6 +60,11 @@ class WikiTestManagerInterface(Component):
     _config_properties = {}
     sortby = 'custom'
     open_new_window = False
+    ticket_summary_option = 'full_path'
+    ticket_summary_option_values = ['full_path', 'last_n_catalogs', 'empty', 'fixed_text']
+    ticket_summary_text = ''
+    ticket_summary_num_catalogs = '1'
+    ticket_summary_separator = ' - '
     
     def __init__(self, *args, **kwargs):
         """
@@ -80,7 +85,13 @@ class WikiTestManagerInterface(Component):
         if 'testmanager' in self.config:
             self.sortby = self.config.get('testmanager', 'testplan.sortby', 'custom')
             self.open_new_window = self.config.get('testmanager', 'testcase.open_new_window', '') == 'True'
-                        
+            self.ticket_summary_option = self.config.get('testmanager', 'ticket_summary_option', 'full_path')
+            if not (self.ticket_summary_option in self.ticket_summary_option_values):
+                raise TracError("Configuration property 'ticket_summary_option' in trac.ini file must be one of: " + ', '.join(self.ticket_summary_option_values))
+            self.ticket_summary_text = self.config.get('testmanager', 'ticket_summary_text', '')
+            self.ticket_summary_num_catalogs = self.config.get('testmanager', 'ticket_summary_num_catalogs', '')
+            self.ticket_summary_separator = self.config.get('testmanager', 'ticket_summary_separator', ' - ')
+
     # IWikiChangeListener methods
     def wiki_page_added(self, page):
         """Called whenever a new Wiki page is added."""
@@ -253,8 +264,12 @@ class WikiTestManagerInterface(Component):
                         fieldLabel,
                         tag.span(id='catErrorMsgSpan', style='color: red;'),
                         tag.br(),
-                        tag.input(id='catName', type='text', name='catName', size='50'),
-                        tag.input(type='button', value=buttonLabel, onclick='creaTestCatalog("'+cat_name+'")')
+                        tag.form(id='createTestCatalogForm', action='%s/testcreate' % fix_base_location(req), method='post')(
+                            tag.input(type='hidden', name='type', value='catalog'),
+                            tag.input(type='hidden', name='path', value=cat_name),
+                            tag.input(type='text', name='title', size='50'),
+                            tag.input(type='submit', value=_(buttonLabel))
+                            )
                         )
                     ))
 
@@ -266,16 +281,28 @@ class WikiTestManagerInterface(Component):
                             _("New Test Case:"),
                             tag.span(id='errorMsgSpan', style='color: red;'),
                             tag.br(),
-                            tag.input(id='tcName', type='text', name='tcName', size='50'),
-                            tag.input(type='button', value=_("Add a Test Case"), onclick='creaTestCase("'+cat_name+'")')
+                            tag.form(id='createTestCaseForm', action='%s/testcreate' % fix_base_location(req), method='post')(
+                                tag.input(type='hidden', name='type', value='testcase'),
+                                tag.input(type='hidden', name='path', value=cat_name),
+                                tag.input(type='text', name='title', size='50'),
+                                tag.input(type='submit', value=_("Add a Test Case"))
+                                )
                             ),
                         tag.br(), 
                         tag.label(
                             _("New Test Plan:"),
                             tag.span(id='errorMsgSpan2', style='color: red;'),
                             tag.br(),
-                            tag.input(id='planName', type='text', name='planName', size='50'),
-                            tag.input(type='button', value=_("Generate a new Test Plan"), onclick='creaTestPlan("'+cat_name+'")')
+                            tag.form(id='createTestPlanForm', action='%s/testcreate' % fix_base_location(req), method='post')(
+                                tag.input(type='hidden', name='mustSubmit', value='false'),
+                                tag.input(type='hidden', name='type', value='testplan'),
+                                tag.input(type='hidden', name='path', value=cat_name),
+                                tag.input(type='hidden', name='containsAll', value=''),
+                                tag.input(type='hidden', name='snapshot', value=''),
+                                tag.input(type='hidden', name='selectedTCs', value=''),
+                                tag.input(type='text', name='title', size='50'),
+                                tag.input(type='submit', value=_("Generate a new Test Plan"))
+                                )
                             ),
                         tag.br(), 
                         ))
@@ -602,33 +629,83 @@ class WikiTestManagerInterface(Component):
         result += '<table><tbody>'
         
         for f in fields:
-            result += "<tr onmouseover='showPencil(\"field_pencilIcon"+f['name']+"\", true)' onmouseout='hidePencil(\"field_pencilIcon"+f['name']+"\", false)'>"
+            field_name = f['name']
+            field_value = obj[field_name]
+            field_type = f['type']
             
-            if f['type'] == 'text':
-                result += '<td><label for="custom_field_'+f['name']+'">'+f['label']+':</label></td>'
-                
-                result += '<td>'
-                result += '<span id="custom_field_value_'+f['name']+'" name="custom_field_value_'+f['name']+'">'
-                if obj[f['name']] is not None:
-                    result += obj[f['name']]
-                result += '</span>'
+            result += "<tr class='customFieldContainer' onmouseover='showEditingPencil(\""+field_name+"\")' onmouseout='hidePencil(\"field_pencilIcon"+field_name+"\")'>"
             
-                result += '<input style="display: none;" type="text" id="custom_field_'+f['name']+'" name="custom_field_'+f['name']+'" '
-                if obj[f['name']] is not None:
-                    result += ' value="' + obj[f['name']] + '" '
+            result += '<td><label for="custom_field_'+field_name+'">'+f['label']+':</label></td>'
+            
+            result += '<td>'
+            result += '<span id="custom_field_value_'+field_name+'" name="custom_field_value_'+field_name+'">'
+            if field_value is not None:
+                if field_type == 'textarea':
+                    temp_contents = html_escape(field_value)
+                    result += temp_contents.replace('\n\r', '<br />').replace('\n', '<br />')
+                else:
+                    result += html_escape(field_value)
+                    
+            result += '</span><span id="container_custom_field_'+field_name+'" style="display: none;">'
+            
+            if field_type == 'text':
+                result += '<input type="text" id="custom_field_'+field_name+'" name="custom_field_'+field_name+'" '
+                if field_value is not None:
+                    result += ' value="' + html_escape(field_value) + '" '
                 result += '></input>'
-                result += '</td>'
+                
+            elif field_type == 'textarea':
+                result += '<textarea id="custom_field_'+field_name+'" name="custom_field_'+field_name+'" '
+                if f['width'] is not None:
+                    result += ' cols="' + str(f['width']) + '" '
+                if f['height'] is not None:
+                    result += ' rows="' + str(f['height']) + '" '
+                    
+                result += '>'
+                    
+                if field_value is not None:
+                    result += html_escape(field_value)
+                result += '</textarea>'
+                
+            elif field_type == 'radio':
+                if f['options'] is not None and len(f['options']) > 0:
+                    i = 0
+                    for option in f['options']:
+                        result += '<input type="radio" name="custom_field_'+field_name+'" '
+                        result += ' value="'+html_escape(option)+'" '
+                        
+                        if option == field_value:
+                            result += ' checked '
+                        
+                        result += '>' + html_escape(option) + '</input>'
+                        i += 1
+                
+            elif field_type == 'select':
+                if f['options'] is not None and len(f['options']) > 0:
+                    result += '<select id="custom_field_'+field_name+'" name="custom_field_'+field_name+'">'
 
-                result += '<td>'
-                result += '<span class="rightIcon" style="display: none;" title="'+_("Edit")+'" onclick="editField(\''+f['name']+'\')" id="field_pencilIcon'+f['name']+'"></span>'
-                result += '</td>'
+                    for option in f['options']:
+                        result += '<option value="'+html_escape(option)+'" '
+                        
+                        if option == field_value:
+                            result += ' selected '
+                        
+                        result += '>' + html_escape(option) + '</option>'
 
-                result += '<td>'
-                result += '<input style="display: none;" type="button" onclick="sendUpdate(\''+obj.realm+'\', \'' + f['name']+'\')" id="update_button_'+f['name']+'" name="update_button_'+f['name']+'" value="'+_("Save")+'"></input>'
-                result += '</td>'
+                    result += '</select>'
+                
+            elif field_type == 'checkbox':
+                result += '<input type="checkbox" id="custom_field_'+field_name+'" name="custom_field_'+field_name+'" '
+                if field_value is not None and (str(field_value) == '1'):
+                    result += ' checked '
+                result += '></input>'
+                
+            result += '</span>'
+
+            result += '<span class="rightIcon" style="display: none;" title="'+_("Edit")+'" onclick="editField(\''+field_name+'\', \''+field_type+'\')" id="field_pencilIcon'+field_name+'"></span>'
+            result += '<input style="display: none;" type="button" onclick="sendUpdate(\''+obj.realm+'\', \'' + field_name+'\', \''+field_type+'\')" id="update_button_'+field_name+'" name="update_button_'+field_name+'" value="'+_("Save")+'"></input>'
+            result += '</td>'
 
-            # TODO Support other field types
-            
             result += '</tr>'
 
         result += '</tbody></table>'
@@ -758,8 +835,8 @@ class WikiTestManagerInterface(Component):
             text += '<td>'+format_datetime(from_any_timestamp(ts))+'</td>'
             text += '<td>'+html_escape(author)+'</td>'
             text += '<td>'+fname+'</td>'
-            text += '<td>'+oldvalue+'</td>'
-            text += '<td>'+newvalue+'</td>'
+            text += '<td>'+html_escape(oldvalue)+'</td>'
+            text += '<td>'+html_escape(newvalue)+'</td>'
             text += '</tr>'
             
         text += '</tbody></table>'
@@ -993,10 +1070,15 @@ class WikiTestManagerInterface(Component):
         after_jquery = u'var jQuery_testmanager = $.noConflict(true);$ = jQuery_trac_old;jQuery = $;'
 
         if add_statuses_and_colors and 'TEST_EXECUTE' in req.perm:
-            after_jquery += "var buildStatusChangeMenu = true;"
+            after_jquery += "var buildStatusChangeMenu = true;\n"
         else:
-            after_jquery += "var buildStatusChangeMenu = false;"
-        
+            after_jquery += "var buildStatusChangeMenu = false;\n"
+
+        after_jquery += "var ticketSummaryOption = '%s';\n" % self.ticket_summary_option
+        after_jquery += "var ticketSummaryText = '%s';\n" % self.ticket_summary_text
+        after_jquery += "var ticketSummaryNumCatalogs = '%s';\n" % self.ticket_summary_num_catalogs
+        after_jquery += "var ticketSummarySeparator = '%s';\n" % self.ticket_summary_separator
+
         common_code = tag.div()(
             HTML(self._get_error_dialog_markup(req)),
             tag.script(before_jquery, type='text/javascript'),
@@ -1181,7 +1263,6 @@ class WikiTestManagerInterface(Component):
         
         markup, num_plans = self._render_testplan_list(catid, mode, fulldetails, show_delete_button)
 
-
         text = u'<form id="testPlanList" class="printableform"><fieldset id="testPlanListFields" class="collapsed"><legend class="foldable" style="cursor: pointer;"><a href="#no4"  onclick="expandCollapseSection(\'testPlanListFields\')">'+_("Available Test Plans")+' ('+str(num_plans)+')</a></legend>'
         text +='<div style="padding: 0px 0px 10px 10px">'+_("Filter:")+' <input id="tpFilter" title="'+_("Type the test to search for, even more than one word.")+'" type="text" size="40" onkeyup="starthighlightTable(\'testPlanListTable\', this.value)"/>&nbsp;&nbsp;<span id="testPlanListTable_searchResultsNumberId" style="font-weight: bold;"></span></div>'
         text += markup
@@ -1237,7 +1318,7 @@ class WikiTestManagerInterface(Component):
                 plan_param = plan_ref
         
             text += '<span name="breadcrumb" style="cursor: pointer; color: #BB0000; margin-left: 5px; margin-right: 5px; font-size: 0.8em;" '
-            text += ' onclick="window.location=\''+x['id']+'?mode='+mode+plan_param+'&fulldetails='+str(fulldetails)+'\'">'+x['title']
+            text += ' onclick="window.location=\''+x['id']+'?mode='+mode+plan_param+'&fulldetails='+str(fulldetails)+'\'">'+html_escape(x['title'])
             
             if i < path_len-1:
                 text += '</span><span style="color: #BB0000; margin-left: 2px; margin-right: 2px;">->'
@@ -1252,6 +1333,13 @@ class WikiTestManagerInterface(Component):
     def _render_subtree(self, planid, component, ind, level):
         data = component
         text = u''
+
+        has_status = planid is not None and planid != '-1'
+        if has_status:
+            plan_param = '?planid='+planid
+        else:
+            plan_param = ''
+
         if (level == 0):
             data = component['childrenC']
             text +='<ul style="list-style: none;">';
@@ -1263,48 +1351,51 @@ class WikiTestManagerInterface(Component):
             ind['count'] += 1
             text+='<li style="font-weight: normal">'
             comp = data[x]
-            if ('childrenC' in comp):
-                subcData=comp['childrenC']
-                
-                toggle_icon = '../chrome/testmanager/images/plus.png'
-                toggable = 'toggable'
-                if (len(comp['childrenC']) + len(comp['childrenT'])) == 0:
-                    toggable = 'nope'
-                    toggle_icon = '../chrome/testmanager/images/empty.png'
+            
+            if not has_status or comp['tot'] > 0:
+                if 'childrenC' in comp:
+                    subcData=comp['childrenC']
                     
-                index = str(ind['count'])
+                    toggle_icon = '../chrome/testmanager/images/plus.png'
+                    toggable = 'toggable'
+                    if (len(comp['childrenC']) + len(comp['childrenT'])) == 0:
+                        toggable = 'nope'
+                        toggle_icon = '../chrome/testmanager/images/empty.png'
+                        
+                    index = str(ind['count'])
 
-                has_status = True
-                if planid is not None and not planid == '-1':
-                    plan_param = '?planid='+planid
-                    color = [comp['color'], 'yellow'][comp['color'] == 'none']
-                    statusIcon='../chrome/testmanager/images/folder-%s.png' % color
-                else:
-                    has_status = False
-                    plan_param = ''
+                    if has_status:
+                        color = [comp['color'], 'yellow'][comp['color'] == 'none']
+                        statusIcon='../chrome/testmanager/images/folder-%s.png' % color
 
-                text += '<span name="'+toggable+'" style="cursor: pointer" id="b_'+index+'"><span onclick="toggle(\'b_'+index+'\')"><img class="iconElement" src="'+toggle_icon+'" /></span>'
+                    text += '<span name="'+toggable+'" style="cursor: pointer" id="b_'+index+'"><span onclick="toggle(\'b_'+index+'\')"><img class="iconElement" src="'+toggle_icon+'" /></span>'
 
-                # Include color icon for the aggregated status of all sub catalogs/test cases
-                if has_status:
-                    text += '<img class="aggregatedStatusIconElement" style="cursor: default;" src="'+statusIcon+'"></img>'
+                    # Include color icon for the aggregated status of all sub catalogs/test cases
+                    if has_status:
+                        text += '<img class="aggregatedStatusIconElement" style="cursor: default;" src="'+statusIcon+'"></img>'
 
-                text += '<span id="l_'+index+'" onmouseover="underlineLink(\'l_'+index+'\')" onmouseout="removeUnderlineLink(\'l_'+index+'\')" onclick="window.location=\''+comp['id']+plan_param+'\'" title="'+_("Open")+'">'+comp['title']+'</span></span><span style="color: gray;">&nbsp;('+str(comp['tot'])+')</span>'
+                    text += '<span id="l_'+index+'" onmouseover="underlineLink(\'l_'+index+'\')" onmouseout="removeUnderlineLink(\'l_'+index+'\')" onclick="window.location=\''+comp['id']+plan_param+'\'" title="'+_("Open")+'">'+comp['title']+'</span></span><span style="color: gray;">&nbsp;('+str(comp['tot'])+')</span>'
 
-                text +='<ul id="b_'+index+'_list" style="display:none;list-style: none;">';
-                ind['count']+=1
-                text += self._render_subtree(planid, subcData, ind, level+1)
-                if ('childrenT' in comp):            
-                    mtData=comp['childrenT']
-                    text += self._render_testcases(planid, mtData)
-            text+='</ul>'
+                    text +='<ul id="b_'+index+'_list" style="display:none;list-style: none; margin-bottom: 0px;">';
+                    ind['count']+=1
+                    text += self._render_subtree(planid, subcData, ind, level+1)
+                    if ('childrenT' in comp):            
+                        mtData=comp['childrenT']
+                        text += self._render_testcases(planid, mtData)
+                    text+='</ul>'
+                
             text+='</li>'
+            
         if (level == 0):
             if ('childrenT' in component):            
                 cmtData=component['childrenT']
                 text += self._render_testcases(planid, cmtData)
-            text+='</ul>'        
-        return text
+            text+='</ul>'
+            
+        if has_status and ind['count'] == 0:
+            return u''
+        else:
+            return text
 
     def _render_testcases(self, planid, data): 
         
@@ -1339,11 +1430,11 @@ class WikiTestManagerInterface(Component):
                     statusLabel = tc_statuses[status][1]
             
                 tcid = tick['id'].rpartition('TC')[2]
-                text+='<li name="tc_node" style="font-weight: normal;"><span class="statusMenuAnchor" name="'+tcid+','+planid+','+tick["id"]+','+status+','+stat_meaning+','+statusLabel+'"><img id="statusIcon'+tick["id"]+'" class="statusIconElement" src="'+statusIcon+'" title="'+statusLabel+'" style="cursor: pointer;"></img></span><span onmouseover="showPencil(\'pencilIcon'+tick["id"]+'\', true)" onmouseout="hidePencil(\'pencilIcon'+tick["id"]+'\', false)"><a href="'+tick["id"]+'?planid='+planid+version_str+'" '+tc_target+'>'+tick["title"]+'&nbsp;</a><span style="display: none;">'+statusLabel+'</span><span><a class="rightIcon" style="display: none;" title="'+_('Edit the Test Case')+'" href="'+tick["id"]+'?action=edit&planid='+planid+'" '+tc_target+' id="pencilIcon'+tick["id"]+'"></a></span></span>'
+                text+='<li name="tc_node" style="font-weight: normal; margin-left: 10px;"><span class="statusMenuAnchor" name="'+tcid+','+planid+','+tick["id"]+','+status+','+stat_meaning+','+statusLabel+'"><img id="statusIcon'+tick["id"]+'" class="statusIconElement" src="'+statusIcon+'" title="'+statusLabel+'" style="cursor: pointer;"></img></span><span onmouseover="showPencil(\'pencilIcon'+tick["id"]+'\', true)" onmouseout="hidePencil(\'pencilIcon'+tick["id"]+'\', false)"><a href="'+tick["id"]+'?planid='+planid+version_str+'" '+tc_target+'>'+tick["title"]+'&nbsp;</a><span style="display: none;">'+statusLabel+'</span><span><a class="rightIcon" style="display: none;" title="'+_('Edit the Test Case')+'" href="'+tick["id"]+'?action=edit&planid='+planid+'" '+tc_target+' id="pencilIcon'+tick["id"]+'"></a></span></span>'
 
                 text+='</li>'
             else:
-                text+='<li name="tc_node" style="font-weight: normal;"><input name="select_tc_checkbox" value="'+tick["id"]+'" type="checkbox" style="display: none;float: left; position: relative; top: 3px;" /><span onmouseover="showPencil(\'pencilIcon'+tick["id"]+'\', true)" onmouseout="hidePencil(\'pencilIcon'+tick["id"]+'\', false)"><a href="'+tick["id"]+"?a=a"+version_str+'" '+tc_target+'>'+tick["title"]+'&nbsp;</a><span><a class="rightIcon" style="display: none;" title="'+_('Edit the Test Case')+'" href="'+tick["id"]+'?action=edit" '+tc_target+' id="pencilIcon'+tick["id"]+'"></a></span></span></li>'
+                text+='<li name="tc_node" style="font-weight: normal; margin-left: 10px;"><input name="select_tc_checkbox" value="'+tick["id"]+'" type="checkbox" style="display: none;float: left; position: relative; top: 3px;" /><span onmouseover="showPencil(\'pencilIcon'+tick["id"]+'\', true)" onmouseout="hidePencil(\'pencilIcon'+tick["id"]+'\', false)"><a href="'+tick["id"]+"?a=a"+version_str+'" '+tc_target+'>'+tick["title"]+'&nbsp;</a><span><a class="rightIcon" style="display: none;" title="'+_('Edit the Test Case')+'" href="'+tick["id"]+'?action=edit" '+tc_target+' id="pencilIcon'+tick["id"]+'"></a></span></span></li>'
                 
         return text
             
@@ -1839,13 +1930,14 @@ class WikiTestManagerInterface(Component):
         
         for f in fields:
             if table_columns_map[f['name']]['visible'] == 'True':
-                if f['type'] == 'text':
-                    result += '<td>'
-                    if obj[f['name']] is not None:
-                        result += obj[f['name']]
-                    result += '</td>'
-
-                # TODO Support other field types
+                result += '<td>'
+                if obj[f['name']] is not None:
+                    if f['type'] == 'textarea':
+                        temp_contents = html_escape(obj[f['name']])
+                        result += temp_contents.replace('\n\r', '<br />').replace('\n', '<br />')
+                    else:
+                        result += html_escape(obj[f['name']])
+                result += '</td>'
 
         return result
 
